/* Copyright (C) 1999 PLUS Group - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in ParkaNet.java
 * and the file License.html for details.
 */

package ParkaClient;

import java.awt.*;
import java.awt.event.*;
import UserInt.Utilities;

/** A GUI that allows the user to test functionality of the ParkaClient
  code. Currently, the functions Query, Insert Assertions, Delete
  Assertions, Insert Frames, Delete Frames and Insert Predicates are
  supported. */
public class SimpleClient extends Frame 
                        implements ActionListener,
                                   ItemListener,
                                   ParkaWin
{
  /** Default host machine for KB */
  protected static String DEFAULT_SERVER = "";
  /** Default port number to connect to KB */
  protected static String DEFAULT_PORT = "";

  /** Array of commonly accessed KB names. */
  protected static String[] AVAIL_KBS = new String[0];

  protected static final String commands[][] = {
    {"Query", "10003"},
    {"Get KB Names", "10010"},
    {"Insert Assertion", "10013"},
    {"Delete Assertion", "10014"},
    {"Insert Frame", "10015"},
    {"Delete Frame", "10016"},
    {"Get Predicates", "10005"},
    {"Get Predicate Arity", "10021"},            // not yet supported
    {"Insert Predicate", "10022"},
    {"Get Immediate Descendants", "10019"},      // not yet supported
    {"Get Descendant Pairs", "10020"},           // not yet supported
    {"Get Timings", "10011"}, 
    {"Get KB stats", "10009"}
  };


  /* maybe add these commands later */
  /*
 JBROWSERQUERY = "10017";
 JBRWQUERYCONT = "10018";
 GETFRAMENAME  = "10006"; 
 GETFRAMEID    = "10007"; 
 TOGGLE        = "10008"; 
 */


  /* GUI components */
  Button b_Connect;
  Button b_Execute;
  Button b_Quit;
  TextField serverFld;
  TextField portFld;
  Choice kbFld;
  TextField kbText;
  Choice commandFld;
  TextField cmdArgFld;
  TextArea responseFld;

  /* Cursors */
  static final Cursor WAIT_CURSOR = Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR);
  static final Cursor DEFAULT_CURSOR = Cursor.getDefaultCursor();

  ParkaNet pn;


  public static void main(String args[]) {

    SimpleClient app;

    app = new SimpleClient();
  }

  public SimpleClient() {

    setup();
    setSize(getMinimumSize());
    show();
    pn = new ParkaNet(this);
  }

  void setup() {

    int i;

    setLayout(new GridBagLayout());

    // server connection information
    serverFld = new TextField(DEFAULT_SERVER);
    portFld = new TextField(DEFAULT_PORT);
    b_Connect = new Button("Connect");

    kbFld = new Choice();
    for (i=0; i < AVAIL_KBS.length; i++)
      kbFld.add(AVAIL_KBS[i]);
    kbText = new TextField(20);
    if (AVAIL_KBS.length > 0)
      kbText.setText(AVAIL_KBS[0]);

    commandFld = new Choice();
    for (i=0; i < commands.length; i++)
      commandFld.add(commands[i][0]);
    cmdArgFld = new TextField(40);
    b_Execute = new Button("Execute");
    responseFld = new TextArea(15,70);
    b_Quit = new Button("Quit");

    Utilities.placeComponent(this, new Label("Host:"),     0, 0, 1, 1);
    Utilities.placeComponent(this, serverFld,              1, 0, 2, 1);
    Utilities.placeComponent(this, new Label("Port:"),     0, 1, 1, 1);
    Utilities.placeComponent(this, portFld,                1, 1, 2, 1);
    Utilities.placeComponent(this, new Label("KB:"),       0, 2, 1, 1);
    Utilities.placeComponent(this, kbText,                 1, 2, 1, 1);
    Utilities.placeComponent(this, kbFld,                  2, 2, 1, 1);
    Utilities.placeComponent(this, b_Connect,              0, 3, 1, 1);
    Utilities.placeComponent(this, new Label("Command:"),  0, 4, 1, 1);
    Utilities.placeComponent(this, commandFld,             1, 4, 2, 1);
    Utilities.placeComponent(this, new Label("Argument:"), 0, 5, 1, 1);
    Utilities.placeComponent(this, cmdArgFld,              1, 5, 2, 1);
    Utilities.placeComponent(this, new Label("Response:"), 0, 6, 1, 1);
    Utilities.placeComponent(this, b_Execute,              0, 7, 1, 1);
    Utilities.placeComponent(this, responseFld,            0, 8, 3, 1);
    Utilities.placeComponent(this, b_Quit,                 0, 9, 1, 1);

    b_Connect.addActionListener(this);
    kbFld.addItemListener(this);
    // commandFld.addItemListener(this);
    b_Execute.addActionListener(this);
    b_Quit.addActionListener(this);

    pack();
  }

  /** Catches button presses */
  public void actionPerformed(ActionEvent event) {

    Object source;                   // the source of the event
    String parkaHost = null;
    int parkaPort = 0;
    String parkaKB = null;
    int cmdindx;
    String cmdId;
    String arg;
    QueryResults qr;

    source = event.getSource();

    if (source == b_Connect) {
      parkaHost = serverFld.getText();
      try {
	parkaPort = Integer.parseInt(portFld.getText());
      } catch (NumberFormatException ex) {
	System.out.println("Invalid port number");
      }
      // parkaKB = kbfld.getSelectedItem();
      parkaKB = kbText.getText();

      if (parkaHost != null && parkaPort != 0)
	pn.changeParkaHost(parkaHost, parkaPort);
      if (pn.isConnection()) {
	if (parkaKB != null)
	  pn.openKB(parkaKB, true);
      }
      else
	System.out.println("Connection failed");
    }

    else if (source == b_Execute) {
      setCursor (WAIT_CURSOR);
      cmdindx = commandFld.getSelectedIndex();
      cmdId = commands[cmdindx][1];
      arg = cmdArgFld.getText();

      if (cmdId.equals(Command.QUERY)) {
	responseFld.append("Issuing Query: " + arg + "\n");
        qr = pn.query (arg);
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
	if (qr != null) {
	  responseFld.append("--> Results = ");

	  /* display variable names */
	  String names[] = qr.getNames();
	  for (int k = 0; k < names.length; k++) {
	    responseFld.append (names[k] + " ");
	  }
	  responseFld.append("\n");

	  /* display variable bindings */
	  String mat[][] = qr.getValueMatrix();
	  int i, j;
	  for (i=0; i < mat.length; i++) {
	    responseFld.append("\t");
	    for(j=0; j < mat[i].length; j++)
	      responseFld.append(mat[i][j] + ", ");
	    responseFld.append("\n");
	  }
	}
      }

      else if (cmdId.equals(Command.INSERTTOKB)) {
	responseFld.append("Inserting Assertion: " + arg + "\n");
        pn.insertToKB (arg);
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
      }

      else if (cmdId.equals(Command.DELETEFROMKB)) {
	responseFld.append("Deleting Assertion: " + arg + "\n");
        pn.deleteFromKB (arg);
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
      }

      else if (cmdId.equals(Command.INSERTFRAME)) {
	responseFld.append("Inserting Frame: " + arg + "\n");
        pn.insertFrame (arg);
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
      }

      else if (cmdId.equals(Command.DELETEFRAME)) {
	responseFld.append("Deleting Frame: " + arg + "\n");
        pn.deleteFrame (arg);
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
      }

      else if (cmdId.equals(Command.GETTIMINGS)) {

	String[] timeStats;
	int i;

	timeStats = pn.getTimingStats();
	if (timeStats != null) {
	  responseFld.append("Times:\n");
	  for (i=0; i < timeStats.length; i++)
	    responseFld.append("\t" + timeStats[i] + "\n");
	}
	else
	  responseFld.append("Get Timings Errror: " + pn.statusMsg + "\n");
      }

      else if (cmdId.equals(Command.INSERTPREDTOKB)) {
	int spaceloc = arg.indexOf(' ');
	String pred = arg.substring(0, spaceloc);
	int arity = 2;
	try {
	  arity = Integer.parseInt(arg.substring(spaceloc + 1));
	} catch (Exception ex) {
	  System.out.println(arg.substring(spaceloc) + " is not an integer");
	}
	responseFld.append("Inserting predicate: " + pred + 
			   ", arity = " + arity + "\n");
        pn.insertPredicate(pred, arity);
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
      }

      else if (cmdId.equals(Command.GETPREDICATES)) {
	responseFld.append("Getting Predicates:\n");
	String[]  predList = pn.getPreds();
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
        responseFld.append("--> Predicates =\n");

	/* display KB names */
	for (int k = 0; k < predList.length; k++) {
	    responseFld.append ("\t" + predList[k] + "\n");
	}
      }

      else if (cmdId.equals(Command.PARKACONFIG)) {

	String[] kbStats;
	int i;

	kbStats = pn.getKbStats();
	if (kbStats != null) {
	  responseFld.append("KB Stats:\n");
	  for (i=0; i < kbStats.length; i++)
	    responseFld.append("\t" + kbStats[i] + "\n");
	}
	else
	  responseFld.append("Parka Config Errror: " + pn.statusMsg + "\n");
      }

      else if (cmdId.equals(Command.GETKBNAMES)) {
	responseFld.append("Getting KB Names:\n");
	String[]  kblist = pn.getKbNames();
        responseFld.append("--> Status = " + pn.statusMsg + "\n");
        responseFld.append("--> KBs =\n");

	/* display KB names */
	for (int k = 0; k < kblist.length; k++) {
	    responseFld.append ("\t" + kblist[k] + "\n");
	}
      }

      else {
	System.out.println("That command has not been implemented");
      }

    }
    else if (source == b_Quit) {
      pn.disconnect();
      System.exit(0);
    }
    setCursor (DEFAULT_CURSOR);
  }

  /** Listens for updates to the KB choice box */
  public void itemStateChanged(ItemEvent event) {

    Object source;                   // the source of the event
    String parkaKB = null;

    source = event.getSource();
    if (source == kbFld) {
      kbText.setText(kbFld.getSelectedItem());
      /*
      parkaKB = kbFld.getSelectedItem();
      if (pn.isConnection()) {
	if (parkaKB != null)
	  pn.openKB(parkaKB, true);
      }
      */
    }
    else if (source == commandFld) {
      ;
    }
  }

  /** Implment ParkaWin interface methods */
  public void displayMessage(String msg) {

    responseFld.append(msg + "\n");
  }
}
