/* Copyright (C) 1999 Jeff Heflin - University of Maryland
 *	
 * This software is provided under the terms of the GNU Lesser General Public
 * License Version 2.1, February 1999. See the header in ShoeKb.java
 * and the file License.html for details.
 */

package ShoeKb;

import java.util.*;

/** An inference rule (or axiom) used to conclude a fact from a given
  set of facts. Only Horn-clause style rules are supported, that is,
  there is a conjunction of antecedents and a single consequent. */
/* NOTE: This should really be a sublcass of Sentence.java, where
   conjuncts are treated as consequents. */
public class Axiom {

  Vector antecedents;
  Vector consequents;

  /** Create a new rule with no antecedents or consequents. */
  public Axiom() {

    antecedents = new Vector();
    consequents = new Vector();
  }

  /** Create a new rule with a single antecedent and consequent. */
  public Axiom(Atom theAntecedent, Atom theConsequent) {

    antecedents = new Vector();
    consequents = new Vector();
    antecedents.addElement(theAntecedent);
    consequents.addElement(theConsequent);
  }

  /** Returns the antecedents of the axiom. */
  public Vector getAntecedents() {

    return antecedents;
  }

  /** Returns the consequents of the axiom */
  public Vector getConsequents() {

    return consequents;
  }

  /** Adds an antecedent to the existing conjunction of antecedents. */
  public void addAntecedent(Atom theAntecedent) {

    antecedents.addElement(theAntecedent);
  }

  /** Adds an antecedent to the existing conjunction of antecedents. */
  public void addAntecedent(String predicate, String arg1, String arg2) {

    Atom theAntecedent;

    theAntecedent = new Atom(predicate, arg1, arg2);
    antecedents.addElement(theAntecedent);
  }

  /** Adds a consequent to the existing conjunction of consequents. */
  public void addConsequent(Atom theConsequent) {

    consequents.addElement(theConsequent);
  }


  /** Adds a consequent to the existing conjunction of consequents. */
  public void addConsequent(String predicate, String arg1, String arg2) {
    
    Atom theConsequent;

    theConsequent = new Atom(predicate, arg1, arg2);
    consequents.addElement(theConsequent); 
  }

  public String toString() {

    StringBuffer sbuf;
    Enumeration  antEnum;
    Enumeration  conEnum;
    Atom         conjunct;

    sbuf = new StringBuffer();
    antEnum = antecedents.elements();
    while (antEnum.hasMoreElements()) {
      conjunct = (Atom)antEnum.nextElement();
      sbuf.append(conjunct.toString());
    }

    sbuf.append("->");

    conEnum = consequents.elements();
    while (conEnum.hasMoreElements()) {
      conjunct = (Atom)conEnum.nextElement();
      sbuf.append(conjunct.toString());
    }
    return sbuf.toString();
  }
}
